#include <bits/stdc++.h>

using namespace std;

/**
    Algorithm      : merge-intervals + segment-tree (for queries decomposition)
    Expected score : 60p
    Author         : Alexandru Văleanu
**/

const int NIL = -1;

struct AdjacencyList
{
    struct Edge
    {
        int v;
        int urm;

        Edge() : v(0), urm(0) {
        }

        Edge(const int _v, const int _urm) : v(_v), urm(_urm) {
        }
    };

    int *head;
    Edge *lista;
    int contor;

    AdjacencyList(const int N, const int M)
    {
        head = new int[N];
        lista = new Edge[M];
        contor = 0;

        for (int i = 0; i < N; ++i)
            head[i] = NIL;
    }

    void addEdge(int x, int y) /// x->y
    {
        lista[contor] = {y, head[x]};
        head[x] = contor++;
    }
};

const int MAX_N = 50000;
const int MAX_P = 5000;
const int MAX_Q = 30000;
const int MAX_S = 300000 + 1;

bitset <MAX_P> queryXOR[MAX_Q];

int longArray[MAX_S];
int startPosition[MAX_N];
int lengthArmy[MAX_N];

int tmpArray[MAX_P];

AdjacencyList listQueries(4 * MAX_N, MAX_Q * 18);

int indexArmy[4 * MAX_N];

int N, Q;

void build(int nod, int st, int dr)
{
    if (st == dr)
    {
        indexArmy[nod] = st;
    }
    else
    {
        int m = (st + dr) / 2;

        build(2 * nod, st, m);
        build(2 * nod + 1, m + 1, dr);
    }
}

void addQuery(int nod, int st, int dr, const int l, const int r, const int indQuery)
{
    if (l <= st && dr <= r)
    {
        listQueries.addEdge(nod, indQuery);
    }
    else
    {
        int m = (st + dr) / 2;

        if (l <= m)
            addQuery(2 * nod, st, m, l, r, indQuery);

        if (m < r)
            addQuery(2 * nod + 1, m + 1, dr, l, r, indQuery);
    }
}

int solution[MAX_N];
bitset<MAX_P> BitSet;

void solveQueries(int nod, int st, int dr)
{
    if (st != dr)
    {
        int mid = (st + dr) / 2;

        solveQueries(2 * nod, st, mid);
        solveQueries(2 * nod + 1, mid + 1, dr);

        BitSet.reset();

        int cnt = 0;

        int i = startPosition[ indexArmy[2 * nod] ];
        int j = startPosition[ indexArmy[2 * nod + 1] ];
        int n = i + lengthArmy[ indexArmy[2 * nod] ];
        int m = j + lengthArmy[ indexArmy[2 * nod + 1] ];

        while (i < n && j < m)
        {
            int e1 = longArray[i];
            int e2 = longArray[j];

            if (e1 == e2)
            {
                i++;
                j++;
            }
            else if (e1 < e2)
            {
                BitSet.set(e1);

                solution[ cnt++ ] = e1;
                i++;
            }
            else
            {
                BitSet.set(e2);

                solution[ cnt++ ] = e2;
                j++;
            }
        }

        while (i < n)
        {
            BitSet.set(longArray[i]);

            solution[ cnt++ ] = longArray[i];
            i++;
        }

        while (j < m)
        {
            BitSet.set(longArray[j]);

            solution[ cnt++ ] = longArray[j];
            j++;
        }

        indexArmy[nod] = indexArmy[2 * nod];

        int p = startPosition[ indexArmy[nod] ];

        for (i = 0; i < cnt; ++i)
            longArray[p + i] = solution[i];

        lengthArmy[ indexArmy[nod] ] = cnt;
    }
    else
    {
        BitSet.reset();

        int pos = startPosition[ indexArmy[nod] ];

        for (int k = 0; k < lengthArmy[ indexArmy[nod] ]; ++k)
            BitSet.set(longArray[pos + k]);
    }

    for (int p = listQueries.head[nod]; p != NIL; p = listQueries.lista[p].urm)
    {
        int indQ = listQueries.lista[p].v;
        queryXOR[indQ] ^= BitSet;
    }
}

int main()
{
    ifstream in("infinitywar.in");
    ofstream out("infinitywar.out");

    ios_base::sync_with_stdio(false);

    assert(in >> N);
    assert(in >> Q);

    int lastPos = 0;

    for (int i = 0; i < N; ++i)
    {
        int K;
        assert(in >> K);

        for (int j = 0; j < K; ++j)
        {
            int power;
            assert(in >> power);
            assert(power > 0);

            tmpArray[j] = power - 1;
        }

        sort(tmpArray, tmpArray + K);

        startPosition[i] = lastPos;
        lengthArmy[i] = K;

        for (int j = 0; j < K; ++j)
            longArray[ lastPos++ ] = tmpArray[j];
    }

    memset(indexArmy, -1, sizeof(indexArmy));

    build(1, 0, N - 1);

    for (int i = 0; i < Q; ++i)
    {
        int x, y;
        assert(in >> x >> y);
        assert(1 <= x && x <= N);
        assert(1 <= y && y <= N);
        x--; y--;

        addQuery(1, 0, N - 1, x, y, i);
    }

    solveQueries(1, 0, N - 1);

    for (int i = 0; i < Q; ++i)
    {
        int a = queryXOR[i].count();
        out << a << "\n";
    }

    in.close();
    out.close();

    return 0;
}
