/**
 * Time complexity: O(S + (N + Q) * P / 32)
 * Memory: O(S + P + Q), about 6.4 MB
 **/
#include <stdio.h>

#define MAX_N 50000
#define MAX_S 300000
#define MAX_Q 100000
#define MAX_P 10016 /* round up to a multiple of 32 */
#define NIL -1

/* For every power, store a list of armies containing it. */
int pHead[MAX_P];
unsigned short pArmy[MAX_S];
int pNext[MAX_S];
int pTop;

/* Store the queries and compute all the answers in parallel. */
unsigned short qx[MAX_Q], qy[MAX_Q];
short ans[MAX_Q];

/* Partial xors */
unsigned xor[MAX_N];

/* Precomputed 16-bit popcounts */
unsigned char lookup[65536];

/* Returns the number of set bits in x */
__inline__ int popcount(unsigned x) {
  return lookup[x >> 16] + lookup[x & 0xffff];
}

int main() {
  int n, q, i, j, k, p, base;
  unsigned x;

  for (i = 0; i < 65536; i++) {
    lookup[i] = lookup[i / 2] + (i & 1);
  }
  
  FILE *f = fopen("infinitywar.in", "r");
  fscanf(f, "%d%d", &n, &q);

  /* Initialize and read the power lists. */
  for (i = 0; i < MAX_P; i++) {
    pHead[i] = NIL;
  }

  for (i = 0; i < n; i++) {
    fscanf(f, "%d", &k);
    for (j = 0; j < k; j++) {
      fscanf(f, "%d", &p);
      p--;
      pArmy[pTop] = i;
      pNext[pTop] = pHead[p];
      pHead[p] = pTop++;
    }
  }

  /* Read the queries. */
  for (i = 0; i < q; i++) {
    fscanf(f, "%hu%hu", &qx[i], &qy[i]);
    qx[i]--;
    qy[i]--;
  }
  fclose(f);

  /* Process powers in batches of 32 */
  for (base = 0; base < MAX_P; base += 32) {
    /* Clear the partial xors for the batch base ... base+31. */
    for (i = 0; i < n; i++) {
      xor[i] = 0;
    }

    /* Compute the mask for each army, restricted to base ... base+31 */
    for (p = base; p < base + 32; p++) {
      for (i = pHead[p]; i != NIL; i = pNext[i]) {
        xor[pArmy[i]] ^= 1u << (p - base);
      }
    }

    /* Compute partial sums over xor[] */
    for (i = 1; i < n; i++) {
      xor[i] ^= xor[i - 1];
    }

    /* Add the popcounts to the total answers for every query */
    for (i = 0; i < q; i++) {
      x = qx[i] ? (xor[qy[i]] ^ xor[qx[i] - 1]) : xor[qy[i]];
      ans[i] += popcount(x);
    }
  }

  f = fopen("infinitywar.out", "w");
  for (i = 0; i < q; i++) {
    fprintf(f, "%d\n", ans[i]);
  }
  fclose(f);
  return 0;
}
