/**
 * Time complexity: O(S + P*Q)
 * Memory: O(S + P*Q), about 128 MB
 **/
#include <stdio.h>

#define MAX_N 50000
#define MAX_S 300000
#define MAX_Q 100000
#define MAX_P 10016
#define NIL 0xffffffff

/* All armies go into one buffer. Store the starting offset of each army. */
short aData[MAX_S];
int aStart[MAX_N + 1];

/* For each army, store a list of queries refering to it. */
unsigned qHead[MAX_N];
unsigned qArmy[2 * MAX_Q], qNext[2 * MAX_Q];
int qTop;

/* Store bitsets for each query, plus one working bitset. */
unsigned bitSet[1 + MAX_P / 32];
unsigned qAns[MAX_Q][1 + MAX_P / 32];

/* Precomputed 16-bit popcounts */
unsigned char lookup[65536];

/* Toggles the x'th bit. */
__inline__ void bitToggle(int x) {
  bitSet[x >> 5] ^= 1 << (x & 31);
}

/* Xors the working bitset into qAns[q]. */
void massXor(int q) {
  int i;
  for (i = MAX_P >> 5; i >= 0; i--) {
    qAns[q][i] ^= bitSet[i];
  }
}

/* Returns the number of set bits in qAns[q]. */
int popCount(int q) {
  int i, result = 0;
  unsigned x;
  for (i = MAX_P >> 5; i >= 0; i--) {
    x = qAns[q][i];
    result += lookup[x >> 16] + lookup[x & 0xffff];
  }
  return result;
}

int main() {
  int n, q, i, j, k, x, y, p;

  for (i = 0; i < 65536; i++) {
    lookup[i] = lookup[i / 2] + (i & 1);
  }
  
  FILE *f = fopen("infinitywar.in", "r");
  fscanf(f, "%d%d", &n, &q);

  /* Read the armies */
  aStart[0] = 0;
  for (i = 0; i < n; i++) {
    fscanf(f, "%d", &k);
    aStart[i + 1] = aStart[i] + k;
    for (j = 0; j < k; j++) {
      fscanf(f, "%d", &p);
      aData[aStart[i] + j] = p;
    }
  }

  /* Initialize and read the query lists */
  for (i = 0; i < n; i++) {
    qHead[i] = NIL;
  }

  for (i = 0; i < q; i++) {
    fscanf(f, "%d%d", &x, &y);
    x -= 2;
    y--;
    if (x >= 0) {
      qArmy[qTop] = i;
      qNext[qTop] = qHead[x];
      qHead[x] = qTop++;
    }
    qArmy[qTop] = i;
    qNext[qTop] = qHead[y];
    qHead[y] = qTop++;
  }
  fclose(f);

  /* Compute the partial xor of the first i armies and use it to answer */
  /* related queries. */
  for (i = 0; i < n; i++) {
    for (j = aStart[i]; j < aStart[i + 1]; j++) {
      bitToggle(aData[j]);
    }

    for (p = qHead[i]; p != NIL; p = qNext[p]) {
      massXor(qArmy[p]);
    }
  }

  f = fopen("infinitywar.out", "w");
  for (i = 0; i < q; i++) {
    fprintf(f, "%d\n", popCount(i));
  }
  fclose(f);

  return 0;
}
